#line 1 "../Temp_meter.ino"
#include <Arduino.h>
#line 1
/*	===========================================================================
 UPC - EETAC - CSD - http://digsys.upc.edu

 *  P12: FSM with LCD. (phase 1: Preparing a basic FSM))
 *	P12: A/D (phase 2: Digital thermometer)

 * Hardware for prototyping: Arduino Uno board. Chip ATmega328P

* Install the LCD library " LiquidCrystal_PCF8574
"Arduino Library for LiquidCrystal displays with I2C PCF8574 adapter"
https://github.com/mathertel/LiquidCrystal_PCF8574


Planning: 
- Step 1: Study the basic FSM + LCD available at:
	http://digsys.upc.edu/arduino/P12/P12.html
				
- Step 2: Prepare the hardware, so that adapt the Arduino board to this 
particular application. For instance:

    - A/D function. Measure temperature 

- Step 3: Prepare the software. Hardware/software diagram. State diagram

    - Real-time measurement using millis() 
	- Sampling period flag. 
	- Plan the datapath() function to process temperature 

=============================================================================*/

/*	===========================================================================
	Include section
	=========================================================================*/
/*-----( Import needed libraries )-----*/
//#include <Wire.h>           // Comes with Arduino IDE

#include <Wire.h>
#include <LiquidCrystal_PCF8574.h>
void setup ();
void loop ();
void state_logic (void );
void output_logic (void );
void write_outputs (void );
void datapath (void );
void interrupt_service_routine_INT0 (void );
void update_time_flag (void );
#line 39

 // set the LCD address to 0x27 for a 16 chars and 2 line display
LiquidCrystal_PCF8574 lcd(0x27); 


/*	===========================================================================
	Definitions
	======================================================================== */
const char pin_SENSOR		= A1;
const char pin_REFERENCE	= A2;

// Start-stop push-button. PD2 is also the microcontroller's external INT0
const char pin_ST_BUTTON = 2;  // 

// Sample time LED indicator (internal)
const char pin_SP_LED = 8; 
// External LED drains to much current and generates variations of Vcc 
// const char pin_SP_LED = 8; 
// FSM list of statesstate list
const char IDLE					= 'A'; 
const char SET_ACQ_TIMER		= 'B';
const char WAIT_SAMPLE_PULSE_1	= 'C';
const char ACQUIRE_V1			= 'D';
const char WAIT_SAMPLE_PULSE_2	= 'E';
const char ACQUIRE_V2			= 'F';
const char TEMP_MEAS			= 'G';
const char STOP_ACQ_TIMER		= 'H';

// Default timer period to handle V1 and V2 samplings at 2 Hz
const unsigned long TIMER_PERIOD = 250; // 250 ms   (4 Hz)




/*	===========================================================================
	Global RAM variables
	======================================================================== */
char   	var_ST_flag		= 0; // Start/stop flag. Set by interrupt INT0 
char   	var_SP_flag		= 0; // Sampling period flag. Set by real-time counter
							 // to generate samplings
char   	var_LCD_flag	= 0;//Set when there is new information to write to LCD


char   	var_SP_LED	= 0; 


char var_current_state ;    // The internal memory of the FSM 

// For calculating timer pulses
static unsigned long var_previous_time;   

static unsigned long var_timer_period;   

// Voltage variables
static unsigned int var_V_sensor;   
static unsigned int var_V_reference;  
// To save LCD writing cycles when the acquired value is the same
static unsigned int var_V_sensor_prev;   
static unsigned int var_V_reference_prev;  



// Temperature variables
static float var_Temp_sensor;   
static float var_Temp_reference;  

 
/*	===========================================================================
	System init
	=========================================================================*/
void setup() { 

  int error;

// Serial monitor initialisation
	Serial.begin(9600); 
 
    // See http://playground.arduino.cc/Main/I2cScanner how to test for a I2C device.
  Wire.begin();
  Wire.beginTransmission(0x27);
  error = Wire.endTransmission();
  Serial.print("Error: ");
  Serial.print(error);

  if (error == 0) {
    Serial.println(": LCD found.");
   // Init LCD using convenient libraries
   // initialize the lcd for 16 chars 2 lines, turn on backlight
    lcd.begin(16, 2);  // initialize the lcd

  } else {
    Serial.println(": LCD not found.");
  }  // if



// Digital input and output configuration 
	pinMode(pin_SP_LED, OUTPUT);  
	pinMode(pin_ST_BUTTON, INPUT);  


    lcd.setBacklight(255);
    lcd.home();
    lcd.clear();
    lcd.print("Hello LCD");
    delay(1000);

    lcd.setBacklight(0);
    delay(400);
    lcd.setBacklight(255);

//-------- Write characters on the display ------------------
// NOTE: Cursor Position: (CHAR, LINE) 
	lcd.setCursor(0,0); //Start at character 0 on line 0
	lcd.print("Project ET - CSD");
	delay(500);
	lcd.setCursor(0,1);
	lcd.print("LCD thermometer ");
	delay(2000);  // 2 seconds

// Clean screen
	lcd.clear();

//  Allow new LCD messages in the main loop
	var_LCD_flag = 1; 

 //Initialise the FSM and the timer 
	var_current_state = IDLE;	
	var_previous_time = 0; 	 

  //attachInterrupt(interrupt, function, mode) 
	attachInterrupt(0, interrupt_service_routine_INT0, FALLING);


		// Serial monitor:
	Serial.println("Project ET - CSD");
	Serial.println("LCD thermometer for reference and sensor measurements"); 

}

/*	===========================================================================
	Infinite loop 
	======================================================================== */
void loop() {       
	state_logic();    
	output_logic();
}

/*	===========================================================================
	State logic to determine where to go next
	======================================================================== */
void state_logic(void){

	update_time_flag();    //  to keep track of real time

	switch (var_current_state) {
	case IDLE:	// Do nothing, wait for start signal
		if (var_ST_flag == 1) {	  // start the process
			  var_current_state = SET_ACQ_TIMER;
			  var_ST_flag = 0;	  //Clear flag when one used
		}
		  else
			var_current_state = IDLE;     
		break;

	case SET_ACQ_TIMER:	// Do nothing, wait for start signal
		var_current_state = WAIT_SAMPLE_PULSE_1;       
		break;

	case WAIT_SAMPLE_PULSE_1:	// Wait for sampling period
		if (var_SP_flag == 1) {	  // start processing 
			var_current_state = ACQUIRE_V1;
			var_SP_flag = 0;	  //Clear flag when one used
		}
		  else
			var_current_state = WAIT_SAMPLE_PULSE_1;      
		break;

	case ACQUIRE_V1:
		var_current_state = WAIT_SAMPLE_PULSE_2;     
		break;

	case WAIT_SAMPLE_PULSE_2:	// Wait for sampling period
		if (var_SP_flag == 1) {	  // start processing 
			var_current_state = ACQUIRE_V2;
			var_SP_flag = 0;	  //Clear flag when one used
		}
		  else
			var_current_state = WAIT_SAMPLE_PULSE_2;      
		break;

	case ACQUIRE_V2:
		var_current_state = TEMP_MEAS;     
		break;

	case TEMP_MEAS:	
		if (var_ST_flag == 1) {	 //if there is an spot pulse
			  var_current_state = STOP_ACQ_TIMER;
			  var_ST_flag = 0;	  //Clear flag when one used
		}
		  else
			var_current_state = WAIT_SAMPLE_PULSE_1;  	// continue with sampling
		break;

	case STOP_ACQ_TIMER:	// go to Idle to wait for another Start pulse
		var_current_state = IDLE;     
		break;

	}	
}

/*	===========================================================================
	Output logic 
	======================================================================== */
void output_logic(void){

	switch (var_current_state) {
	case IDLE:
		var_SP_LED = 0;
		if (var_LCD_flag == 1){
			lcd.setCursor(0,0); //Start at character 4 on line 0
			lcd.print("..... IDLE .....");
		//	delay(50);	// 50 ms
			lcd.setCursor(0,1);
			lcd.print("Click Start ....");
		//	delay(50);  // 50 ms

		// Serial monitor:
			Serial.println("IDLE state. Click ST/SP button to start and stop"); 

			var_LCD_flag = 0; // Allows new information in main loop		
		}

	
		break;

	case SET_ACQ_TIMER:	
		var_timer_period = TIMER_PERIOD; 
		break;

	case WAIT_SAMPLE_PULSE_1:
		var_SP_LED = var_SP_LED; //This is like doing nothing  		
		break;

	case ACQUIRE_V1: 
		// Toggle operation with a XOR simply to indicate running acqusition
		var_SP_LED = var_SP_LED^0b00000001; 	
		var_V_sensor = analogRead(pin_SENSOR); // read V1 voltage

		if (var_V_sensor != var_V_sensor_prev) 
			var_LCD_flag = 1; // set the flag for writing the LCD

		var_V_sensor_prev = var_V_sensor;


		break;
		
	case WAIT_SAMPLE_PULSE_2:
		var_SP_LED = var_SP_LED; //This is like doing nothing  		
		break;		


	case ACQUIRE_V2: 
		var_V_reference = analogRead(pin_REFERENCE); // read V2 voltage
		
		if (var_V_reference != var_V_reference_prev) 
			var_LCD_flag = 1; // set the flag for writing the LCD

		var_V_reference_prev = var_V_reference;

		break;			

	case TEMP_MEAS:	
		if (var_LCD_flag == 1){
			datapath(); //calculates new temperatures only when new data
			lcd.clear();
			lcd.setCursor(0,0);//Start at character 0 on line 0
			lcd.print("Temp_ref = ");
			lcd.setCursor(12,0); //Start at character 12 on line 0
			lcd.print((float)(var_Temp_reference));
			lcd.setCursor(0,1); //Start at character 0 on line 1
			lcd.print("Temp_sen = ");
			lcd.setCursor(12,1); //Start at character 12 on line 0
			lcd.print((float)var_Temp_sensor);


		// Serial monitor:
		Serial.print("Reference temperature = "); 	
		Serial.println((float)var_Temp_reference, 1); 

		Serial.print("Sensor temperature = "); 	
		Serial.println((float)var_Temp_sensor, 1); 

		var_LCD_flag = 0; // Information written 	
		}
		break;

	case STOP_ACQ_TIMER:
		var_LCD_flag = 1; // Information to write when in IDLE	
		break;
	}

	write_outputs();
	
}

/*	===========================================================================
	write_outputs (this time only the sampling period LED)
	======================================================================== */
void write_outputs(void){
   if (var_SP_LED == 1)
       digitalWrite(pin_SP_LED, HIGH);  
   else
       digitalWrite(pin_SP_LED, LOW);  
}

/*	===========================================================================
	datapath function for processing temperature measurements
		 Parameters: (VREF+ = 5 V == ATmega Power Supply)
		 Temperature range 20(�C) < T(�C) < 50(�C)
		 Sensor LM35 ==> Vs = (10 mV/�C)*T(�C)
		 Amplifier non-inverter gain = 10 to fit VREF+ at 50�C
		 Equation ==> 	var_V_sensor = (1024)*(10mV)*(10)* T(�C)/5 V
						var_V_sensor = 20.48 * T(�C)						
	======================================================================== */
void datapath(void){
int var_buff;

	var_Temp_sensor = var_V_sensor/20.48; 
	var_Temp_reference = var_V_reference/20.48;

// Rounding a float with only one decimal digit
	var_Temp_reference = var_Temp_reference*10 + 0.5;
	var_buff = (int)var_Temp_reference;
	var_Temp_reference = (float)var_buff/10;

	var_Temp_sensor = var_Temp_sensor *10 + 0.5;
	var_buff = (int)var_Temp_sensor ;
	var_Temp_sensor  = (float)var_buff/10;

}





/*	===========================================================================
	ISR external interrupt INT0 pin 
This function is executed when an active edge is detected at Pin PD2 (INT0)
	======================================================================== */
void interrupt_service_routine_INT0 (void){

	var_ST_flag = 1;  
	delay(5);  // To clean signal bounces

}


/*	===========================================================================
	Update time to calculate timer pulses (var_SP_flag indicator) to allow 
	sampling signals alternatively at var_timer_period parameter.
	This function can be replaced by timer peripherals in MPLABX and Michochip
	or Atmel devices to gain accuracy in sampling periods and use of interrupts 
	======================================================================== */
void update_time_flag(void){

static unsigned long var_current_time;    // current time in ms
static unsigned long var_time_elapsed;   //  time elapsed since last flag

  // From internal CLK: 
    var_current_time = millis();
    var_time_elapsed =  var_current_time - var_previous_time;  

	if(var_time_elapsed  > var_timer_period) {
			// is the flag is set is because time elapsed is sampling period
		var_SP_flag = 1;   
			// to start another sampling period cycle
		var_previous_time = var_current_time; 
	}
}
